/*
** Module   : NullField.java
** Abstract : outputs null characters to a stream
**
** Copyright (c) 2006-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ---------------------------Description------------------------------
** 001 GES 20060925   @29904 Created initial version which outputs null
**                           characters to a stream.
** 002 SIY 20071001   @35317 Made class suitable for network transfers and
**                           for code reuse. Ref: #35314
** 003 SIY 20071003   @35349 Header cleanup. Added implementation of 
**                           resolveData() method.
**                           Ref: #35347.
** 004 CA  20130308          Added protected setters for the explicit and num fields.
** 005 OM  20130527          Added int64 support.
**                           Added 16 bit restriction observed in Progress.
*/ 

package com.goldencode.p2j.util;

import java.io.*;

/**
 * Outputs null characters to a stream. 
 */
public class NullField
extends FieldEntry
{
   /** Contains an explicit column position. */
   private boolean explicit = false;
   
   /**
    * An explicit column position.
    * From experimental tests one can observe that Progress handles the number of Fields 
    * (CONTROL NULL, SPACE and SKIP) on a signed 16 bit memory space. Although Progress will 
    * accept any 64-bit value it will be automatically truncate it to short. 
    */
   private short num = 0;
   
   /**
    * Default constructor (outputs a single space).
    */
   public NullField()
   {
   }
   
   /**
    * Constructor for an explicit number of null characters.
    *
    * @param    num
    *           The number of null characters to output.
    */
   public NullField(long num)
   {
      this.explicit = true;
      this.num      = (short) num;
   }
   
   /**
    * Renders the contained field definition to the given stream.
    *
    * @param    out
    *           The stream to which the field should be rendered.
    */
   public void output(Stream out)
   throws ErrorConditionException
   {
      if (explicit)
      {
         out.putNull(num);
      }
      else
      {
         out.putNull();
      }
   }

   /**
    * Notify instance that it is about to be transferred so all resolvable
    * data should be resolved and prepared for sending via network.
    */
   public void resolveData()
   {
      // do nothing
   }
   
   /**
    * Replacement for the default object reading method. The latest
    * state is read from the input source.
    *
    * @param    in
    *           The input source from which fields will be restored.
    *
    * @throws   IOException
    *           In case of I/O errors.
    * @throws   ClassNotFoundException
    *           If payload can't be instantiated.
    */
   public void readExternal(ObjectInput in)
   throws IOException,
          ClassNotFoundException
   {
      explicit = in.readBoolean();
      num      = (short) in.readInt();
   }

   /**
    * Replacement for the default object writing method. The latest
    * state is written to the output destination.
    *
    * @param    out
    *           The output destination to which fields will be saved.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   public void writeExternal(ObjectOutput out)
   throws IOException
   {
      out.writeBoolean(explicit);
      out.writeInt(num);
   }

   /**
    * Get current value of explicit flag.
    * 
    * @return   The current value of flag.
    */
   boolean isExplicit()
   {
      return explicit;
   }

   /**
    * Get explicit count.
    * 
    * @return   The explicit count.
    */
   int getNum()
   {
      return num;
   }

   /**
    * Set explicit count. The value is automatically truncated to 16-bit value.
    * 
    * @param    num
    *           The explicit count.
    */
   protected void setNum(long num)
   {
      this.num = (short) num;
   }
   
   
   /**
    * Set value of explicit flag.
    * 
    * @param    explicit
    *           The value of flag.
    */
   protected void setExplicit(boolean explicit)
   {
      this.explicit = explicit;
   }
}
